'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

// Andromeda theme colors mapped to Basic Dark template structure
const base00 = '#23262E', // Background
base01 = '#D5CED9', // Foreground
base02 = '#db45a280', // Selection and Selection Match
base03 = '#2b303b', // Dropdown Background
base04 = '#FFF', // Cursor
base05 = '#c74ded', // Keyword, Storage
base06 = '#00e8c6', // Variable, Parameter
base07 = '#FFE66D', // Function, Type, Class
base08 = '#96E072', // String, RegExp
base09 = '#ee5d43', // Constant, Number
base0A = '#A0A1A7cc', // Comment
base0B = '#ff00aa', // Heading
base0C = '#f92672', // Tag
invalid = 'red'; // Invalid (fallback color)
const darkBackground = '#292d30', highlightBackground = base02, tooltipBackground = base01, cursor = base04, selection = base02;
// Define the editor theme styles for Andromeda.
const andromedaTheme = view.EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: selection },
    '.cm-panels': { backgroundColor: darkBackground, color: base06 },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: base02,
        outline: `1px solid ${base03}`,
        color: base07,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base00,
        color: base07,
    },
    '.cm-search.cm-panel': {
        color: base06,
    },
    '.cm-selectionMatch': { backgroundColor: highlightBackground },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        outline: `1px solid ${base03}`,
    },
    '&.cm-focused .cm-matchingBracket': {
        backgroundColor: base02,
        color: base07,
    },
    '.cm-gutters': {
        borderRight: '1px solid #ffffff10',
        color: base06,
        backgroundColor: darkBackground,
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: base02,
    },
    '.cm-tooltip': {
        border: 'none',
        backgroundColor: tooltipBackground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: base03,
        },
    },
}, { dark: true });
// Define the highlighting style for code in the Andromeda theme.
const andromedaHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base05 },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.propertyName, highlight.tags.macroName], color: base06 },
    { tag: [highlight.tags.variableName], color: base06 },
    { tag: [highlight.tags.function(highlight.tags.variableName)], color: base05 },
    { tag: [highlight.tags.labelName], color: base09 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base05 },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base07 },
    { tag: [highlight.tags.brace], color: base07 },
    { tag: [highlight.tags.annotation], color: invalid },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: base05 },
    { tag: [highlight.tags.typeName, highlight.tags.className], color: base06 },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base07 },
    { tag: [highlight.tags.tagName], color: base05 },
    { tag: [highlight.tags.squareBracket], color: base07 },
    { tag: [highlight.tags.angleBracket], color: base07 },
    { tag: [highlight.tags.attributeName], color: base06 },
    { tag: [highlight.tags.regexp], color: base05 },
    { tag: [highlight.tags.quote], color: base01 },
    { tag: [highlight.tags.string], color: base08 },
    { tag: highlight.tags.link, color: base0C, textDecoration: 'underline', textUnderlinePosition: 'under' },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.special(highlight.tags.string)], color: base0B },
    { tag: [highlight.tags.meta], color: base09 },
    { tag: [highlight.tags.comment], color: base0A, fontStyle: 'italic' },
    { tag: highlight.tags.monospace, color: base01 },
    { tag: highlight.tags.strong, fontWeight: 'bold', color: base05 },
    { tag: highlight.tags.emphasis, fontStyle: 'italic', color: base06 },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base01 },
    { tag: highlight.tags.special(highlight.tags.heading1), fontWeight: 'bold', color: base01 },
    { tag: highlight.tags.heading1, fontWeight: 'bold', color: base01 },
    { tag: [highlight.tags.heading2, highlight.tags.heading3, highlight.tags.heading4], fontWeight: 'bold', color: base01 },
    { tag: [highlight.tags.heading5, highlight.tags.heading6], color: base01 },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base0B },
    { tag: [highlight.tags.processingInstruction, highlight.tags.inserted], color: base0B },
    { tag: [highlight.tags.contentSeparator], color: base06 },
    { tag: highlight.tags.invalid, color: base02, borderBottom: `1px dotted ${invalid}` },
]);
// Extension to enable the Andromeda theme (both the editor theme and the highlight style).
const andromeda = [
    andromedaTheme,
    language.syntaxHighlighting(andromedaHighlightStyle),
];

exports.andromeda = andromeda;
exports.andromedaHighlightStyle = andromedaHighlightStyle;
exports.andromedaTheme = andromedaTheme;
