import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

// Andromeda theme colors mapped to Basic Dark template structure
const base00 = '#23262E', // Background
base01 = '#D5CED9', // Foreground
base02 = '#db45a280', // Selection and Selection Match
base03 = '#2b303b', // Dropdown Background
base04 = '#FFF', // Cursor
base05 = '#c74ded', // Keyword, Storage
base06 = '#00e8c6', // Variable, Parameter
base07 = '#FFE66D', // Function, Type, Class
base08 = '#96E072', // String, RegExp
base09 = '#ee5d43', // Constant, Number
base0A = '#A0A1A7cc', // Comment
base0B = '#ff00aa', // Heading
base0C = '#f92672', // Tag
invalid = 'red'; // Invalid (fallback color)
const darkBackground = '#292d30', highlightBackground = base02, tooltipBackground = base01, cursor = base04, selection = base02;
// Define the editor theme styles for Andromeda.
const andromedaTheme = /*@__PURE__*/EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: selection },
    '.cm-panels': { backgroundColor: darkBackground, color: base06 },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: base02,
        outline: `1px solid ${base03}`,
        color: base07,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base00,
        color: base07,
    },
    '.cm-search.cm-panel': {
        color: base06,
    },
    '.cm-selectionMatch': { backgroundColor: highlightBackground },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        outline: `1px solid ${base03}`,
    },
    '&.cm-focused .cm-matchingBracket': {
        backgroundColor: base02,
        color: base07,
    },
    '.cm-gutters': {
        borderRight: '1px solid #ffffff10',
        color: base06,
        backgroundColor: darkBackground,
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: base02,
    },
    '.cm-tooltip': {
        border: 'none',
        backgroundColor: tooltipBackground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: base03,
        },
    },
}, { dark: true });
// Define the highlighting style for code in the Andromeda theme.
const andromedaHighlightStyle = /*@__PURE__*/HighlightStyle.define([
    { tag: tags.keyword, color: base05 },
    { tag: [tags.name, tags.deleted, tags.character, tags.propertyName, tags.macroName], color: base06 },
    { tag: [tags.variableName], color: base06 },
    { tag: [/*@__PURE__*/tags.function(tags.variableName)], color: base05 },
    { tag: [tags.labelName], color: base09 },
    { tag: [tags.color, /*@__PURE__*/tags.constant(tags.name), /*@__PURE__*/tags.standard(tags.name)], color: base05 },
    { tag: [/*@__PURE__*/tags.definition(tags.name), tags.separator], color: base07 },
    { tag: [tags.brace], color: base07 },
    { tag: [tags.annotation], color: invalid },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: base05 },
    { tag: [tags.typeName, tags.className], color: base06 },
    { tag: [tags.operator, tags.operatorKeyword], color: base07 },
    { tag: [tags.tagName], color: base05 },
    { tag: [tags.squareBracket], color: base07 },
    { tag: [tags.angleBracket], color: base07 },
    { tag: [tags.attributeName], color: base06 },
    { tag: [tags.regexp], color: base05 },
    { tag: [tags.quote], color: base01 },
    { tag: [tags.string], color: base08 },
    { tag: tags.link, color: base0C, textDecoration: 'underline', textUnderlinePosition: 'under' },
    { tag: [tags.url, tags.escape, /*@__PURE__*/tags.special(tags.string)], color: base0B },
    { tag: [tags.meta], color: base09 },
    { tag: [tags.comment], color: base0A, fontStyle: 'italic' },
    { tag: tags.monospace, color: base01 },
    { tag: tags.strong, fontWeight: 'bold', color: base05 },
    { tag: tags.emphasis, fontStyle: 'italic', color: base06 },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
    { tag: tags.heading, fontWeight: 'bold', color: base01 },
    { tag: /*@__PURE__*/tags.special(tags.heading1), fontWeight: 'bold', color: base01 },
    { tag: tags.heading1, fontWeight: 'bold', color: base01 },
    { tag: [tags.heading2, tags.heading3, tags.heading4], fontWeight: 'bold', color: base01 },
    { tag: [tags.heading5, tags.heading6], color: base01 },
    { tag: [tags.atom, tags.bool, /*@__PURE__*/tags.special(tags.variableName)], color: base0B },
    { tag: [tags.processingInstruction, tags.inserted], color: base0B },
    { tag: [tags.contentSeparator], color: base06 },
    { tag: tags.invalid, color: base02, borderBottom: `1px dotted ${invalid}` },
]);
// Extension to enable the Andromeda theme (both the editor theme and the highlight style).
const andromeda = [
    andromedaTheme,
    /*@__PURE__*/syntaxHighlighting(andromedaHighlightStyle),
];

export { andromeda, andromedaHighlightStyle, andromedaTheme };
